@@ -19,23 +19,27 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <unistd.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
-#include <libgnome/libgnome.h>
-#include "config.h"
 #include "bug-buddy.h"
 #include "distribution.h"
 
 #define d(x)
 
+static char *get_lsb_version (Distribution *distro);
 static char *get_redhat_version (Distribution *distro);
 static char *get_turbolinux_version (Distribution *distro);
 static char *get_debian_version (Distribution *distro);
 static char *get_irix_version (Distribution *distro);
 
+Phylum lsb_phy = {
+	get_lsb_version
+};
+
 Phylum redhat_phy = {
 	get_redhat_version
 };
@@ -53,98 +57,119 @@ Phylum irix_phy = {
 };
 
 
-
-
+DistroVersionFunc lsb_ver =  get_lsb_version;
 DistroVersionFunc redhat_ver = get_redhat_version;
 DistroVersionFunc turbolinux_ver = get_turbolinux_version;
 DistroVersionFunc debian_ver = get_debian_version;
 DistroVersionFunc irix_ver =  get_irix_version;
 
 
-char *
-get_line_from_fd (int fd)
+static char**
+get_lines_from_file (const char *filename)
 {
-	char buf[1024];
-	int pos;
-
-	buf[0] = '\0';
-	for (pos = 0; pos < 1023; pos++)
-		if (read (fd, buf+pos, 1) < 1 ||
-		    buf[pos] == '\n')
-			break;
+	char *contents;
+	gsize length;
+	char **lines;
+	GError *error = NULL;
 	
-	if (pos == 0 && buf[0] == '\0')
-		return NULL;
-
-	buf[pos] = '\0';
-	return g_strdup (buf);
-}
-
-char *
-get_line_from_file (const char *filename)
-{
-	char *retval;
-	int fd;
-
-	g_return_val_if_fail (filename, NULL);
-
-	//if (!g_file_exists (filename))
-	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
-		return NULL;
-
-	fd = open (filename, O_RDONLY);
-	if (fd == -1) {
-		d(g_warning ("Could not open file '%s' for reading", filename));
+	if (!g_file_get_contents (filename, &contents, &length, &error)) {
+		g_free (error);
 		return NULL;
 	}
+
+	lines = g_strsplit (contents, "\n", 0);
+	g_free (contents);
 	
-	retval = get_line_from_fd (fd);
-	close (fd);
-	return retval;
+	return lines;
 }
 
+
+static char *
+get_lsb_version (Distribution *distro)
+{
+	char **lines;
+	char *id = NULL;
+	char *release = NULL;
+	char *codename = NULL;
+	char *name = NULL;
+	int i = 0;
+
+	g_return_val_if_fail (distro, NULL);
+	g_return_val_if_fail (distro->version_file, NULL);
+
+	lines = get_lines_from_file (distro->version_file);
+	while (lines && lines[i]) {
+		if (g_str_has_prefix (lines[i], "DISTRIB_ID=")) 
+			id = lines[i] + 11;
+		else if (g_str_has_prefix (lines[i], "DISTRIB_RELEASE=")) 
+			release = lines[i] + 16;
+		else if (g_str_has_prefix (lines[i], "DISTRIB_CODENAME=")) 
+			codename = lines[i] + 17;
+		i++;
+	}
+	if (id != NULL && release != NULL && codename != NULL)
+		name = g_strdup_printf ("%s %s (%s)", id, release, codename);
+
+	g_strfreev (lines);
+
+	return name;
+}
+
+
+
 static char *
 get_redhat_version (Distribution *distro)
 {
+	char **lines;
+	char *name = NULL;
+
 	g_return_val_if_fail (distro, NULL);
 	g_return_val_if_fail (distro->version_file, NULL);
 
-	return get_line_from_file (distro->version_file);
+	lines = get_lines_from_file (distro->version_file);
+	if (lines != NULL && lines[0] != NULL)
+		name = g_strdup (lines[0]);
+
+	g_strfreev (lines);
+
+	return name;
 }
 
 static char *
 get_turbolinux_version (Distribution *distro)
 {
-	char *tmp, *ret;
+	char **lines;
+	char *name = NULL;
 	g_return_val_if_fail (distro, NULL);
 	g_return_val_if_fail (distro->version_file, NULL);
 
-	tmp = get_line_from_file (distro->version_file);
-	ret = g_strdup_printf ("TurboLinux %s", tmp);
-	g_free (tmp);
-	return ret;
+	lines = get_lines_from_file (distro->version_file);
+	if (lines != NULL && lines[0] != NULL)
+                name = g_strdup_printf ("TurboLinux %s", lines[0]);
+
+	g_strfreev (lines);
+
+	return name;
 }
 
 
 static char *
 get_debian_version (Distribution *distro)
 {
-	char *retval, *version;
+	char **lines;
+	char *name = NULL;
 
 	g_return_val_if_fail (distro, NULL);
 	g_return_val_if_fail (distro->version_file, NULL);
 	g_return_val_if_fail (distro->name, NULL);
 
-	version = get_line_from_file (distro->version_file);
-	if (!version) {
-		d(g_warning ("Could not get distro version"));
-		return NULL;
-	}
-	
-	retval = g_strdup_printf ("%s %s", distro->name, version);
-	g_free (version);
+	lines = get_lines_from_file (distro->version_file);
+	if (lines != NULL && lines[0] != NULL)
+                name = g_strdup_printf ("%s %s", distro->name, lines[0]);
 
-	return retval;
+	g_strfreev (lines);
+
+	return name;
 }
 
 
